import { EntityComponentTypes, EquipmentSlot, ItemComponentTypes, system, world } from "@minecraft/server"

const helmet = {
  "minecraft:turtle_helmet": "ht",
  "minecraft:leather_helmet": "hl",
  "minecraft:chainmail_helmet": "hc",
  "minecraft:iron_helmet": "hi",
  "minecraft:golden_helmet": "hg",
  "minecraft:diamond_helmet": "hd",
  "minecraft:netherite_helmet": "hn",

  "generic": "gh",
  "undefined": "he"
},

chestplate = {
  "minecraft:leather_chestplate": "cl",
  "minecraft:chainmail_chestplate": "cc",
  "minecraft:iron_chestplate": "ci",
  "minecraft:elytra": "el",
  "minecraft:golden_chestplate": "cg",
  "minecraft:diamond_chestplate": "cd",
  "minecraft:netherite_chestplate": "cn",

  "generic": "gc",
  "undefined": "ce"
},

leggings = {
  "minecraft:leather_leggings": "ll",
  "minecraft:chainmail_leggings": "lc",
  "minecraft:iron_leggings": "li",
  "minecraft:golden_leggings": "lg",
  "minecraft:diamond_leggings": "ld",
  "minecraft:netherite_leggings": "ln",

  "generic": "gl",
  "undefined": "le"
},

boots = {
  "minecraft:leather_boots": "bl",
  "minecraft:chainmail_boots": "bc",
  "minecraft:iron_boots": "bi",
  "minecraft:golden_boots": "bg",
  "minecraft:diamond_boots": "bd",
  "minecraft:netherite_boots": "bn",

  "generic": "gb",
  "undefined": "be"
}

let runDurabilityFunction = false;


world.afterEvents.playerSpawn.subscribe(event => {
	runDurabilityFunction = true;
	durabilityFunction()
})

world.afterEvents.playerLeave.subscribe(event => {
	runDurabilityFunction = false;
})

function durabilityFunction() {
	let player = world.getAllPlayers()[0];

	system.runInterval(() => {
		if (runDurabilityFunction) shieldDurability(player);
	}, 5);
	system.runInterval(() => {
		if (runDurabilityFunction) armorDurability(player);
	}, 9);
}

world.afterEvents.entityHurt.subscribe(event => {
  armorDurability(event.hurtEntity)
})

world.afterEvents.entityHitEntity.subscribe(event => {
  armorDurability(event.hitEntity)
})

function shieldDurability(player) {  
  const equippable = player.getComponent(EntityComponentTypes.Equippable),
        offhandItem = equippable?.getEquipment(EquipmentSlot.Offhand),
        offhandItemDurability = offhandItem?.getComponent(ItemComponentTypes.Durability)

  if (offhandItemDurability?.maxDurability !== undefined) {
    player.onScreenDisplay.setTitle(`durabilityShield:Shield: ${offhandItemDurability.maxDurability - offhandItemDurability.damage}/${offhandItemDurability.maxDurability}`)
  } else {
    player.onScreenDisplay.setTitle("durabilityShield:No Shield")
  }
}

function armorDurability(player) {
  let equippable = player.getComponent(EntityComponentTypes.Equippable),
      headItem = equippable?.getEquipment(EquipmentSlot.Head),
      chestItem = equippable?.getEquipment(EquipmentSlot.Chest),
      legItem = equippable?.getEquipment(EquipmentSlot.Legs),
      feetItem = equippable?.getEquipment(EquipmentSlot.Feet),
  
      headItemDurability = headItem?.getComponent(ItemComponentTypes.Durability),
      headItemDurabilityText = ` ${headItemDurability?.maxDurability - headItemDurability?.damage}/${headItemDurability?.maxDurability}`,
      chestItemDurability = chestItem?.getComponent(ItemComponentTypes.Durability),
      chestItemDurabilityText = ` ${chestItemDurability?.maxDurability - chestItemDurability?.damage}/${chestItemDurability?.maxDurability}`,
      legItemDurability = legItem?.getComponent(ItemComponentTypes.Durability),
      legItemDurabilityText = ` ${legItemDurability?.maxDurability - legItemDurability?.damage}/${legItemDurability?.maxDurability}`,
      feetItemDurability = feetItem?.getComponent(ItemComponentTypes.Durability),
      feetItemDurabilityText = ` ${feetItemDurability?.maxDurability - feetItemDurability?.damage}/${feetItemDurability?.maxDurability}`,

      headItemName, chestItemName, legItemName, feetItemName

  headItemDurabilityText = headItemDurabilityText == ' NaN/undefined' ? '' : headItemDurabilityText
  chestItemDurabilityText = chestItemDurabilityText == ' NaN/undefined' ? '' : chestItemDurabilityText
  legItemDurabilityText = legItemDurabilityText == ' NaN/undefined' ? '' : legItemDurabilityText
  feetItemDurabilityText = feetItemDurabilityText == ' NaN/undefined' ? '' : feetItemDurabilityText

  headItemName = Object.keys(helmet).includes(`${headItem?.typeId}`) ? helmet[headItem?.typeId] : helmet.generic
  chestItemName = Object.keys(chestplate).includes(`${chestItem?.typeId}`) ? chestplate[chestItem?.typeId] : chestplate.generic
  legItemName = Object.keys(leggings).includes(`${legItem?.typeId}`) ? leggings[legItem?.typeId] : leggings.generic
  feetItemName = Object.keys(boots).includes(`${feetItem?.typeId}`) ? boots[feetItem?.typeId] : boots.generic

  player.onScreenDisplay.setTitle(`durabilityArmor:${headItemName}${chestItemName}${legItemName}${feetItemName}§z${headItemDurabilityText}\n${chestItemDurabilityText}\n${legItemDurabilityText}\n${feetItemDurabilityText}`)
}